<?php
namespace WTFIAN;
require_once('PHP-Parser/lib/bootstrap.php');


use PhpParser;
use PhpParser\Node;


//Looks through code to find certain interesting functions
class findInterestingFunctions extends \PhpParser\NodeVisitorAbstract
{
	private $inFunction = array();
	private $filename = '';
	private $initFunctions = Array('admin_init', 'init', 'wp_loaded', 'wp_enqueue_scripts', 'wp_head', 'wp_print_styles', 'wp_print_scripts', 'wp_footer',
		'plugins_loaded', 'admin_enqueue_scripts', 'admin_print_styles', 'admin_print_scripts', 'admin_head', 'admin_notices', 'admin_head', 'admin_menu',
		'admin_notices', 'user_admin_notices', 'all_admin_notices');
	private $interestingActions = array();
	private $allFunctions = array();
	public function enterNode(Node $node)
	{
		$this->trackCurrentFunction($node);
	}
	public function leaveNode(Node $node)
	{
		$this->trackCurrentFunction($node);
		
		//Look at all the function calls
		if ($node instanceof Node\Expr\FuncCall)
		{
			//We only care about add_action function calls
			if ($node->name->parts[0] == "add_action")
			{
				//Actions we care about
				if (strpos($node->args[0]->value->value, 'wp_ajax_') === 0 || in_array($node->args[0]->value->value, $this->initFunctions))
				{
					//For OO callbacks
					if (isset($node->args[1]->value->items[1]->value->value))
						$this->interestingActions[] = array('hook' => $node->args[0]->value->value, 'callback' => $node->args[1]->value->items[1]->value->value,
							'hookFile' => $this->filename, 'hookLine' => $node->getLine(), 'hookParent' => $this->inFunction);
					//For non OO callbacks
					else
						$this->interestingActions[] = array('hook' => $node->args[0]->value->value, 'callback' => $node->args[1]->value->value,
							'hookFile' => $this->filename, 'hookLine' => $node->getLine(), 'hookParent' => $this->inFunction);
				}
			}
		}
	}
	private function trackCurrentFunction(Node &$node)
	{
		if ($node instanceof PhpParser\Node\Stmt\Function_ || $node instanceof PhpParser\Node\Stmt\ClassMethod)
		{
			if (count($this->inFunction) == 0)
			{
				$this->inFunction = array('name' => $node->name, 'file' => $this->filename, 'line' => $node->getLine());
				//Keep a copy of this function for later. We might want to look at it.
				$this->allFunctions[] = array('name' => $node->name, 'node' => &$node, 'file' => $this->filename, 'line' => $node->getLine());
			}
			else
			{
				$this->inFunction = array();
			}
		}
	}
	public function setFilename($filename)
	{
		$this->filename = $filename;
	}
	public function &getInterestingActions()
	{
		return $this->interestingActions;
	}
	public function &getAllFunctions()
	{
		return $this->allFunctions;
	}
}


//Examine the functions previously identified as being interesting
class examineInterestingFunction extends PhpParser\NodeVisitorAbstract
{
	private $checkNonceFunctions = array('check_admin_referer', 'check_ajax_referer');
	private $checkNonceFunctions2 = array('wp_verify_nonce');
	private $checkCapabilitiesFunctions = array('current_user_can', 'user_can', 'current_user_can_for_blog');
	private $createNonceFunctions = array('wp_nonce_field', 'wp_create_nonce');
	private $createNonceFunctions2 = array('wp_nonce_url');
	private $checkedNonces = array();
	private $createdNonces = array();
	private $checkedCapabilities = array();

        public function leaveNode(Node $node)
        {
		if ($node instanceof Node\Expr\FuncCall)
		{
			//Horrible stuff where we check for functions in 200 parts because they have different parameters
			if (in_array($node->name->parts[0], $this->checkNonceFunctions))
				$this->checkedNonces[] = array('function' => $node->name->parts[0], 'nonce' => $node->args[0]->value->value);
			else if (in_array($node->name->parts[0], $this->checkNonceFunctions2))
				$this->checkedNonces[] = array('fuction' => $node->name->parts[0], 'nonce' => $node->args[1]->value->value);
			else if (in_array($node->name->parts[0], $this->checkCapabilitiesFunctions))
				$this->checkedCapabilities[] = array('function' => $node->name->parts[0], 'capability' => $node->args[0]->value->value);
			else if (in_array($node->name->parts[0], $this->createNonceFunctions))
				$this->createdNonces[] = array('function' => $node->name->parts[0], 'nonce' => $node->args[0]->value->value);
			else if (in_array($node->name->parts[0], $this->createNonceFunctions2))
				$this->createdNonces[] = array('function' => $node->name->parts[0], 'nonce' => $node->args[1]->value->value);
		}
        }
	public function getCheckedNonces()
	{
		return $this->checkedNonces;
	}
	public function getCreatedNonces()
	{
		return $this->createdNonces;
	}
	public function getCheckedCapabilities()
	{
		return $this->checkedCapabilities;
	}
	public function clear()
	{
		$this->checkedNonces = array();
		$this->createdNonces = array();
		$this->checkedCapabilities = array();
	}
}
