<?php
namespace WTFIAN;

use PhpParser;

require_once('PHP-Parser/lib/bootstrap.php');
require_once('nodeVisitors.php');

class action
{
	private $traverser = null;
	private $examineInteresting = null;
	private $hook = array();
	private $callback = array();

	//takes a horrible array of info
	public function __construct($info)
	{
		$this->traverser = new PhpParser\NodeTraverser;
		$this->examineInteresting = new examineInterestingFunction;
		$this->hook['name'] = $info['hook'];
		$this->hook['line'] = $info['hookLine'];
		$this->hook['file'] = $info['hookFile'];
		$this->hook['parent'] = $info['hookParent'];
		$this->callback['name'] = $info['callback'];
	}

	public function getName()
	{
		return $this->callback['name'];
	}
	
	public function getCallbackData()
	{
		return $this->callback;
	}

	public function getHookData()
	{
		return $this->hook;
	}

	public function scan(&$functionList)
	{
		//Add our examiner visitor
		$this->traverser->addVisitor($this->examineInteresting);
		foreach ($functionList as &$function)
		{
			if ($function['name'] == $this->callback['name'])
			{
				$this->traverser->traverse($function);
				$this->callback['checkedNonces'] = $this->examineInteresting->getCheckedNonces();
				$this->callback['createdNonces'] = $this->examineInteresting->getCreatedNonces();
				$this->callback['checkedCapabilities'] = $this->examineInteresting->getCheckedCapabilities();
				$this->callback['nonceSources'] = array();
				$this->callback['line'] = $function['node']->getLine();
				$this->callback['file'] = $function['file'];
				break;
			}
		}
		//remove our traverser
		$this->traverser->removeVisitor($this->examineInteresting);
	}

	public function matchNonces(&$otherAction)
	{
		//see if this action provides a nonce for the other action
		if (count($this->callback['createdNonces']) != 0)
		{
			foreach ($this->callback['createdNonces'] as $createdNonce)
			{
				foreach ($otherAction->callback['checkedNonces'] as $checkedNonce)
				{
					if ($createdNonce['nonce'] == $checkedNonce['nonce'])
					{
						$otherAction->callback['nonceSources'][] = array('nonce' => $createdNonce['nonce'], 'source' => $this->callback['name']);
						break;
					}
				}
			}
		}
		//same thing but reversed
		if (count($this->callback['checkedNonces']) != 0)
		{
			foreach ($this->callback['checkedNonces'] as $checkedNonce)
			{
				foreach ($otherAction->callback['createdNonces'] as $createdNonce)
				{
					if ($checkedNonce['nonce'] == $createdNonce['nonce'])
					{
						$this->callback['nonceSources'][] = array('nonce' => $createdNonce['nonce'], 'source' => $otherAction->callback['name']);
						break;
					}
				}
			}
		}
	}

}

class project
{
	private $parser    = null; //new PhpParser\Parser(new PhpParser\Lexer);
	private $traverser = null; //new PhpParser\NodeTraverser;
	private $findInteresting = null; //new findInterestingFunctions;

	private $fileList = array();
	private $actionList = array();
	private $functionNodes = array();

	public function getActionList()
	{
		return $this->actionList;
	}


	public function __construct()
	{
		$this->parser = new PhpParser\Parser(new PhpParser\Lexer);
		$this->traverser = new PhpParser\NodeTraverser;
		$this->findInteresting = new findInterestingFunctions;
	}

	public function setDirectory($directory)
	{
		//add all the php files to the list
		$dirIt = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($directory));
		foreach ($dirIt as $file)
		{
			if ($file->getExtension() != 'php' || !$file->isFile())
				continue;
			$this->fileList[] = $file->getPathname();
		}
	}

	//Scanning can be performed separately in stages or all at once
	public function scan()
	{
		$this->findActions();
		$this->examineActions();
		$this->matchNonces();
	}

	public function findActions()
	{
		$this->traverser->addVisitor($this->findInteresting);
		foreach ($this->fileList as $file)
		{
			$this->findInteresting->setFilename($file);
			$this->traverser->traverse($this->parser->parse(file_get_contents($file)));
		}

		//make action instances from nasty arrays from findInteresting
		foreach ($this->findInteresting->getInterestingActions() as $action)
			$this->actionList[] = new action($action);

		//remove our visitor
		$this->traverser->removeVisitor($this->findInteresting);
	}

	public function examineActions()
	{
		if (count($this->actionList) == 0)
			return false;

		//scan the interesting functions we found earlier
		foreach ($this->actionList as &$action)
			$action->scan($this->findInteresting->getAllFunctions());
	}

	public function matchNonces()
	{
		if (count($this->actionList) == 0)
			return false;

		for ($i = 0; $i < count($this->actionList); $i++)
		{
			for ($e = $i; $e < count($this->actionList); $e++)
			{
				$this->actionList[$i]->matchNonces($this->actionList[$e]);
			}
		}
	}

	//Pretty is subjective of course
	public function prettyPrint()
	{
		$output = "Interesting Functions:\n";
		foreach ($this->actionList as $action)
		{
			$callback = $action->getCallbackData();
			$hook = $action->getHookData();
			$noncesChecked = implode(',', array_column($callback['checkedNonces'], 'nonce'));
			$noncesCreated = implode(',', array_column($callback['createdNonces'], 'nonce'));
			$capabilitiesChecked = implode(',', array_column($callback['checkedCapabilities'], 'capability'));
			$output .= <<<EOT
Function Name: {$callback['name']}
Hook Name: {$hook['name']}
Hook File: {$hook['file']}
Hook Line: {$hook['line']}
Function File: {$callback['file']}
Function Line: {$callback['line']}
Nonces Checked: {$noncesChecked}
Nonces Created: {$noncesCreated}
Capabilities Checked: {$capabilitiesChecked}

EOT;
			//Handle nonce sources
			if (count($callback['nonceSources']) != 0)
			{
				$output .= "Nonce Sources: \n";
				foreach ($callback['nonceSources'] as $source)
				{
					$output .= "\tNonce '{$source['nonce']}' is generated in function '{$source['source']}'\n";
				}
			}
			//And finally the interesting part
			if (strpos($hook['name'], 'wp_ajax_') === 0)
			{
				if (count($callback['checkedNonces']) == 0 && count($callback['checkedCapabilities']) == 0)
					$output .= "NOTE: May be vulnerable to Authorization Bypass\n";
				if (count($callback['checkedNonces']) == 0)
					$output .= "NOTE: May be vulnerable to CSRF\n";
				if (count($callback['nonceSources']) != 0)
					$output .= "NOTE: Check for nonce leakage\n";
			}
			$output .= "\n\n";
		}
		return $output;
	}
}
